@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     9-Dec-1999
-# RCS-ID:      $Id: buttons.py,v 1.12.2.1 2006/01/12 18:30:15 RD Exp $
+# RCS-ID:      $Id: buttons.py 48466 2007-08-30 19:53:20Z RD $
 # Copyright:   (c) 1999 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -31,8 +31,8 @@ import imageutils
 
 class GenButtonEvent(wx.PyCommandEvent):
     """Event sent from the generic buttons when the button is activated. """
-    def __init__(self, eventType, ID):
-        wx.PyCommandEvent.__init__(self, eventType, ID)
+    def __init__(self, eventType, id):
+        wx.PyCommandEvent.__init__(self, eventType, id)
         self.isDown = False
         self.theButton = None
 
@@ -56,15 +56,15 @@ class GenButton(wx.PyControl):
 
     labelDelta = 1
 
-    def __init__(self, parent, ID, label,
+    def __init__(self, parent, id=-1, label='',
                  pos = wx.DefaultPosition, size = wx.DefaultSize,
                  style = 0, validator = wx.DefaultValidator,
                  name = "genbutton"):
         cstyle = style
         if cstyle == 0:
             cstyle = wx.BORDER_NONE
-        wx.PyControl.__init__(self, parent, ID, pos, size, cstyle, validator, name)
-
+        wx.PyControl.__init__(self, parent, id, pos, size, cstyle, validator, name)
+        
         self.up = True
         self.hasFocus = False
         self.style = style
@@ -77,7 +77,7 @@ class GenButton(wx.PyControl):
 
         self.SetLabel(label)
         self.InheritAttributes()
-        self.SetBestFittingSize(size)
+        self.SetInitialSize(size)
         self.InitColours()
 
         self.Bind(wx.EVT_LEFT_DOWN,        self.OnLeftDown)
@@ -90,17 +90,27 @@ class GenButton(wx.PyControl):
         self.Bind(wx.EVT_KEY_DOWN,         self.OnKeyDown)
         self.Bind(wx.EVT_KEY_UP,           self.OnKeyUp)
         self.Bind(wx.EVT_PAINT,            self.OnPaint)
+        self.InitOtherEvents()
+
+    def InitOtherEvents(self):
+        """
+        Override in a subclass to initialize any other events that
+        need to be bound.  Added so __init__ doesn't need to be
+        overriden, which is complicated with multiple inheritance
+        """
+        pass
 
 
-    def SetBestSize(self, size=None):
+    def SetInitialSize(self, size=None):
         """
         Given the current font and bezel width settings, calculate
         and set a good size.
         """
         if size is None:
             size = wx.DefaultSize            
-        wx.PyControl.SetBestFittingSize(self, size)
-
+        wx.PyControl.SetInitialSize(self, size)
+    SetBestSize = SetInitialSize
+    
 
     def DoGetBestSize(self):
         """
@@ -197,7 +207,9 @@ class GenButton(wx.PyControl):
         self.InitColours()
 
     def SetDefault(self):
-        self.GetParent().SetDefaultItem(self)
+        tlw = wx.GetTopLevelParent(self)
+        if hasattr(tlw, 'SetDefaultItem'):
+            tlw.SetDefaultItem(self)
         
     def _GetLabelSize(self):
         """ used internally """
@@ -255,16 +267,25 @@ class GenButton(wx.PyControl):
         dc.DrawRectangle(bw+2,bw+2,  w-bw*2-4, h-bw*2-4)
         dc.SetLogicalFunction(wx.COPY)
 
-
     def OnPaint(self, event):
         (width, height) = self.GetClientSizeTuple()
         x1 = y1 = 0
         x2 = width-1
         y2 = height-1
-        
-        dc = wx.BufferedPaintDC(self)
-        brush = None
-        
+
+        dc = wx.PaintDC(self)
+        brush = self.GetBackgroundBrush(dc)
+        if brush is not None:
+            dc.SetBackground(brush)
+            dc.Clear()
+
+        self.DrawBezel(dc, x1, y1, x2, y2)
+        self.DrawLabel(dc, width, height)
+        if self.hasFocus and self.useFocusInd:
+            self.DrawFocusIndicator(dc, width, height)
+
+
+    def GetBackgroundBrush(self, dc):
         if self.up:
             colBg = self.GetBackgroundColour()
             brush = wx.Brush(colBg, wx.SOLID)
@@ -283,15 +304,10 @@ class GenButton(wx.PyControl):
                     colBg = self.GetParent().GetBackgroundColour()
                     brush = wx.Brush(colBg, wx.SOLID)
         else:
+            # this line assumes that a pressed button should be hilighted with
+            # a solid colour even if the background is supposed to be transparent
             brush = wx.Brush(self.faceDnClr, wx.SOLID)
-        if brush is not None:
-            dc.SetBackground(brush)
-            dc.Clear()
-                    
-        self.DrawBezel(dc, x1, y1, x2, y2)
-        self.DrawLabel(dc, width, height)
-        if self.hasFocus and self.useFocusInd:
-            self.DrawFocusIndicator(dc, width, height)
+        return brush
 
 
     def OnLeftDown(self, event):
@@ -312,8 +328,9 @@ class GenButton(wx.PyControl):
             if not self.up:    # if the button was down when the mouse was released...
                 self.Notify()
             self.up = True
-            self.Refresh()
-            event.Skip()
+            if self:           # in case the button was destroyed in the eventhandler
+                self.Refresh()
+                event.Skip()
 
 
     def OnMotion(self, event):
@@ -335,29 +352,25 @@ class GenButton(wx.PyControl):
 
     def OnGainFocus(self, event):
         self.hasFocus = True
-        dc = wx.ClientDC(self)
-        w, h = self.GetClientSizeTuple()
-        if self.useFocusInd:
-            self.DrawFocusIndicator(dc, w, h)
+        self.Refresh()
+        self.Update()
 
 
     def OnLoseFocus(self, event):
         self.hasFocus = False
-        dc = wx.ClientDC(self)
-        w, h = self.GetClientSizeTuple()
-        if self.useFocusInd:
-            self.DrawFocusIndicator(dc, w, h)
+        self.Refresh()
+        self.Update()
 
 
     def OnKeyDown(self, event):
-        if self.hasFocus and event.KeyCode() == ord(" "):
+        if self.hasFocus and event.GetKeyCode() == ord(" "):
             self.up = False
             self.Refresh()
         event.Skip()
 
 
     def OnKeyUp(self, event):
-        if self.hasFocus and event.KeyCode() == ord(" "):
+        if self.hasFocus and event.GetKeyCode() == ord(" "):
             self.up = True
             self.Notify()
             self.Refresh()
@@ -369,7 +382,7 @@ class GenButton(wx.PyControl):
 class GenBitmapButton(GenButton):
     """A generic bitmap button."""
 
-    def __init__(self, parent, ID, bitmap,
+    def __init__(self, parent, id=-1, bitmap=wx.NullBitmap,
                  pos = wx.DefaultPosition, size = wx.DefaultSize,
                  style = 0, validator = wx.DefaultValidator,
                  name = "genbutton"):
@@ -377,7 +390,7 @@ class GenBitmapButton(GenButton):
         self.bmpFocus = None
         self.bmpSelected = None
         self.SetBitmapLabel(bitmap)
-        GenButton.__init__(self, parent, ID, "", pos, size, style, validator, name)
+        GenButton.__init__(self, parent, id, "", pos, size, style, validator, name)
 
 
     def GetBitmapLabel(self):
@@ -444,11 +457,11 @@ class GenBitmapButton(GenButton):
 
 class GenBitmapTextButton(GenBitmapButton):
     """A generic bitmapped button with text label"""
-    def __init__(self, parent, ID, bitmap, label,
+    def __init__(self, parent, id=-1, bitmap=wx.NullBitmap, label='',
                  pos = wx.DefaultPosition, size = wx.DefaultSize,
                  style = 0, validator = wx.DefaultValidator,
                  name = "genbutton"):
-        GenBitmapButton.__init__(self, parent, ID, bitmap, pos, size, style, validator, name)
+        GenBitmapButton.__init__(self, parent, id, bitmap, pos, size, style, validator, name)
         self.SetLabel(label)
 
 
@@ -554,7 +567,7 @@ class __ToggleMixin:
         event.Skip()
 
     def OnKeyUp(self, event):
-        if self.hasFocus and event.KeyCode() == ord(" "):
+        if self.hasFocus and event.GetKeyCode() == ord(" "):
             self.up = not self.up
             self.Notify()
             self.Refresh()
@@ -577,4 +590,50 @@ class GenBitmapTextToggleButton(__ToggleMixin, GenBitmapTextButton):
 
 #----------------------------------------------------------------------
 
+class ThemedGenButton(GenButton):
+    " A themed generic button, and base class for the other themed buttons "
 
+    def InitOtherEvents(self):
+        self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouse)
+        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouse)
+        
+    def OnMouse(self, evt):
+        self.Refresh()
+        evt.Skip()
+
+    def DrawBezel(self, dc, x1, y1, x2, y2):
+        rect = wx.Rect(x1, y1, x2, y2)
+        if self.up:
+            state = 0
+        else:
+            state = wx.CONTROL_PRESSED
+        if not self.IsEnabled():
+            state = wx.CONTROL_DISABLED
+        pt = self.ScreenToClient(wx.GetMousePosition())
+        if self.GetClientRect().Contains(pt):
+            state = wx.CONTROL_CURRENT
+        wx.RendererNative.Get().DrawPushButton(self, dc, rect, state)
+        
+ 
+class ThemedGenBitmapButton(ThemedGenButton, GenBitmapButton):
+    """A themed generic bitmap button."""
+    pass
+
+class ThemedGenBitmapTextButton(ThemedGenButton, GenBitmapTextButton):
+    """A themed generic bitmapped button with text label"""
+    pass
+    
+class ThemedGenToggleButton(ThemedGenButton, GenToggleButton):
+    """A themed generic toggle button"""
+    pass
+
+class ThemedGenBitmapToggleButton(ThemedGenButton, GenBitmapToggleButton):
+    """A themed generic toggle bitmap button"""
+    pass
+
+class ThemedGenBitmapTextToggleButton(ThemedGenButton, GenBitmapTextToggleButton):
+    """A themed generic toggle bitmap button with text label"""
+    pass
+
+
+#----------------------------------------------------------------------
